import { useState, useCallback, useMemo, useRef, useEffect } from 'react';
import { doesIdentifierMatchKeyboardEvent } from '../utils/doesIdentifierMatchKeyboardEvent.js';
import { noop } from '../utils/noop.js';

const defaultOptions = {
    eventTypes: ["keydown"],
    when: true,
};
/**
 * useKeyRef hook
 *
 * Fires a callback on keyboard events like keyDown, keyPress and keyUp
 *
 * @param {[string|number]} keys List of keys to listen for. Eg: ["a", "b"]
 * @param {Function} callback Callback to fire on keyboard events
 * @param {Options} options Options
 * @returns {CallbackRef} CallbackRef
 * @see https://rooks.vercel.app/docs/useKeyRef
 */
function useKeyRef(keys, callback, options) {
    const [targetNode, setTargetNode] = useState(null);
    const ref = useCallback((node) => {
        setTargetNode(node);
    }, []);
    const keyList = useMemo(() => {
        if (Array.isArray(keys)) {
            return keys;
        }
        else {
            return [keys];
        }
    }, [keys]);
    const internalOptions = useMemo(() => {
        return Object.assign(Object.assign({}, defaultOptions), options);
    }, [options]);
    const { when, eventTypes } = internalOptions;
    const callbackRef = useRef(callback);
    useEffect(() => {
        callbackRef.current = callback;
    });
    const handle = useCallback((event) => {
        if (keyList.some((identifier) => doesIdentifierMatchKeyboardEvent(event, identifier))) {
            callbackRef.current(event);
        }
    }, [keyList]);
    useEffect(() => {
        if (when && targetNode) {
            for (const eventType of eventTypes) {
                // eslint-disable-next-line prefer-arrow-callback
                targetNode.addEventListener(eventType, handle);
            }
            return () => {
                for (const eventType of eventTypes) {
                    targetNode.removeEventListener(eventType, handle);
                }
            };
        }
        return noop;
    }, [targetNode, when, eventTypes, keyList, handle]);
    return ref;
}

export { useKeyRef };
